@@ -1,4 +1,4 @@
-# $Header: /home/cvsroot/NetZ3950/Z3950/ResultSet.pm,v 1.1.1.1 2001/02/12 10:53:55 mike Exp $
+# $Header: /home/cvsroot/NetZ3950/Z3950/ResultSet.pm,v 1.2 2001/10/12 15:16:13 mike Exp $
 
 package Net::Z3950::ResultSet;
 use strict;
@@ -31,21 +31,26 @@ via the C<Net::Z3950::Connection> class's C<resultSet()> method.
 =cut
 
 
-# Private enumeration for non-reference values of slots in the
-# $this->{records} array.  The slot contains a record reference if we
-# have the record (either because it was piggy-backed in the initial
-# search response, or because we've subsequently got it in a present
-# response); and it's undefined or not there at all (off the end of the
-# array) if we don't have it, and it's not been requested yet):
-sub CALLER_REQUESTED { 1 }	# caller asked for it
-sub RS_REQUESTED { 2 }		# ... and we've issued a Present request
+# The key data member of Result Sets is $this->{records}, which is a
+# hash mapping element-set names to caches of records represented in
+# that element set.  Each such cache is an array, the elements of
+# which may contain any of the following values:
+#	an undefined value (or not there at all -- off the end of the
+#		array) if we don't have the record, and it's not been
+#		requested yet.
+#	CALLER_REQUESTED if the caller has asked for the record but we
+#		don't have it and have not yet issued a Present
+#		request for it.
+#	RS_REQUESTED if the caller has asked for the record and we
+#		don't have it, but we have issued a Present request
+#		and are awaiting a response.
+#	a record reference if we have the record.
+#	a surrogate diagnostic if we fetched the record
+#		unsuccessfully.
 # We use the slots in $this->{records} corresponding to 1-based record
 # numbers; that is, slot zero is not used at all.
-#
-#   ###	Bugger!  This cache doesn't make a distinction between fetches
-#	with different element-sets, so that if you ask for a "b"
-#	record, then ask for the full version, this code will just
-#	give you its cached brief record again.
+sub CALLER_REQUESTED { 1 }
+sub RS_REQUESTED { 2 }
 
 # PRIVATE to the Net::Z3950::Connection class's _dispatch() method
 sub _new {
@@ -55,7 +60,7 @@ sub _new {
     if (!$searchResponse->searchStatus()) {
 	# Search failed: set $conn's error indicators and return undef
 	my $records = $searchResponse->records()
-	    or die "no diagnostics";
+	    or die "no diagnostics (I don't think that's legal)";
 	ref $records eq 'Net::Z3950::APDU::DefaultDiagFormat'
 	    or die "non-default diagnostic format";
 	### $rec->diagnosticSetId() is not used
@@ -68,10 +73,10 @@ sub _new {
 	conn => $conn,
 	rsName => $rsName,
 	searchResponse => $searchResponse,
-	records => [],
+	records => {},
     }, $class;
 
-    ### Should check presentStatus
+    ### Should also check presentStatus where relevant
     my $rawrecs = $searchResponse->records();
     $this->_insert_records($searchResponse, 1, 1)
 	if defined $rawrecs;
@@ -120,7 +125,12 @@ sub record {
     my $this = shift();
     my($which) = @_;
 
-    my $records = $this->{records};
+    my $esn = $this->option('elementSetName');
+    if (!defined $this->{records}->{$esn}) {
+	#warn "setting [] for '$esn'";
+	$this->{records}->{$esn} = [];
+    }
+    my $records = $this->{records}->{$esn};
     my $rec = $records->[$which];
 
     if (ref $rec && $rec->isa('Net::Z3950::APDU::DefaultDiagFormat')) {
@@ -141,50 +151,26 @@ sub record {
 	$this->{conn}->{idleWatcher}->start();
     }
 
-    if ($this->option('mode') ne 'sync') {
+    if ($this->option('mode') eq 'async') {
 	$this->{errcode} = 0;
 	return undef;
     }
 
-    # Synchronous-mode request for a record that we don't yet have: we
-    # need to wait for it to arrive, then return it.  The remainder of
-    # this code is lifted and modified from Net::Connection::search()
-    # which suggests there should be an underlying abstraction?  All
-    # of this would work better with callbacks.
-    #
-    my $xconn = $this->{conn};
-    my $conn = $xconn->manager()->wait();
-    if ($conn != $xconn) {
-	#   ###	We would prefer just to ignore any events on
-	#	connections other than this one, but there doesn't
-	#	seem to be a way to do this (unless we invent one);
-	#	so, for now, you shouldn't mix synchronous and
-	#	asynchronous calls unless the async ones nominate a
-	#	callback (which they can't yet do)
-	die "single-plexing wait() returned wrong connection!";
-    }
-
-    if ($conn->op == Net::Z3950::Op::Error) {
-	# Error code and addinfo are in $conn: copy them across
-	$this->{errcode} = $conn->{errcode};
-	$this->{addinfo} = $conn->{addinfo};
+    # Synchronous-mode request for a record that we don't yet have.
+    # As soon as we're idle -- in the wait() call -- the _idle()
+    # watcher will send a presentRequest; we then wait for its
+    # response to arrive.
+    if (!$this->{conn}->expect(Net::Z3950::Op::Get, "get")) {
+	# Error code and addinfo are in the connection: copy them across
+	$this->{errcode} = $this->{conn}->{errcode};
+	$this->{addinfo} = $this->{conn}->{addinfo};
 	return undef;
     }
 
-    if ($conn->op() != Net::Z3950::Op::Get) {
-	#   ###	Again, we'd like to ignore this event, but there's no
-	#	way to do it, so this has to be a fatal error.
-	die "single-plexing wait() fired wrong op (expected get)";
-    }
-
-    ### We should check that the presentResponse was to this
-    #	particular present response, but then we only get here if
-    #	we're in synchronous mode, so I think it's a "can't happen".
-
-    # OK, the callback invoked by the event loop should now have
-    # inserted the requested record into out array, so we should just
-    # be able to return it.  Sanity-check first, though.
-    die "impossible: didn;t get record" if !defined$records->[$which];
+    # The _add_records() callback invoked by the event loop should now
+    # have inserted the requested record into out array, so we should
+    # just be able to return it.  Sanity-check first, though.
+    die "impossible: didn't get record" if !defined $records->[$which];
     return $this->record(@_);
 }
 
@@ -210,7 +196,8 @@ sub _idle {
 sub _checkRequired {
     my $this = shift();
 
-    my $records = $this->{records};
+    my $esn = $this->option('elementSetName');
+    my $records = $this->{records}->{$esn};
     my $n = @$records;
 
     ###	If our interface to the C function makePresentRequest allowed
@@ -262,12 +249,14 @@ sub _send_presentRequest {
     my($first, $howmany) = @_;
 
     my $refId = _bind_refId($this->{rsName}, $first, $howmany);
+    my $errmsg = '';
     my $pr = Net::Z3950::makePresentRequest($refId,
 				       $this->{rsName},
 				       $first, $howmany,
 				       $this->option('elementSetName'),
-				       $this->option('preferredRecordSyntax'));
-    die "can't make present request" if !defined $pr;
+				       $this->option('preferredRecordSyntax'),
+				       $errmsg);
+    die "can't make present request: $errmsg" if !defined $pr;
     $this->{conn}->_enqueue($pr);
 }
 
@@ -291,7 +280,8 @@ sub _add_records {
     }
 
     if ($this->_insert_records($presentResponse, $first, $howmany)) {
-	my $records = $this->{records};
+	my $esn = $this->option('elementSetName');
+	my $records = $this->{records}->{$esn};
 	for (my $i = $n; $i < $howmany; $i++) {
 	    # We asked for this record but didn't get it, for whatever
 	    # reason.  Mark the record down to "requested by the user
@@ -300,7 +290,7 @@ sub _add_records {
 	    ###	This might not always be The Right Thing -- if the
 	    #	error is a permanent one, we'll end up looping, asking
 	    #	for it again and again.  We could further overload the
-	    #	meaning of numbers in the $this->{records} array to
+	    #	meaning of numbers in the {records}->{$esn} array to
 	    #	count how many times we've tried, and bomb out after
 	    #	"too many" tries.
 	    $this->_check_slot($records->[$first+$i], $first+$i);
@@ -323,7 +313,8 @@ sub _insert_records {
     my($apdu, $first, $howmany) = @_;
     # $first is 1-based; $howmany is used only when storing NSDs.
 
-    my $records = $this->{records};
+    my $esn = $this->option('elementSetName');
+    my $records = $this->{records}->{$esn};
     my $rawrecs = $apdu->records();
     if ($rawrecs->isa('Net::Z3950::APDU::DefaultDiagFormat')) {
 	# Now what?  We want to report the error back to the caller,
@@ -421,13 +412,6 @@ sub _unbind_refId {
 }
 
 
-#   ###	The following records() method is a simplifying interface for
-#	synchronous applications; there should be a similarly
-#	synchronous interface for fetching single records; or perhaps
-#	that's what record() should do if the connection is
-#	synchronous.
-
-
 =head2 records()
 
 	@records = $rs->records();
@@ -467,21 +451,24 @@ caveats to be used extensively in serious applications.
 
 =cut
 
-#   ###	We'd like to do this by just returning $rs->{records} of
-#	course, but we can't do that because (A) it's 1-based, and (B)
-#	we need undefined slots where errors occur rather than
-#	error-information APDUs.  So we make a copy.
+# We'd like to do this by just returning {records}->{$esn} of course, but
+# we can't do that because (A) it's 1-based, and (B) we need undefined
+# slots where errors occur rather than error-information APDUs.  So we
+# make a copy.
 #
 #   ###	It would be nice to come up with some cuter logic for when we
 #	can fall out of our calling-wait()-to-get-more-records loop,
 #	but for now, the trivial keep-going-till-we-have-them-all
 #	approach is adequate.
 #
+#   ###	Does this work?  Does anyone use it?
+#
 sub records {
     my $this = shift();
 
     my $size = $this->size();
-    my $records = $this->{records};
+    my $esn = $this->option('elementSetName');
+    my $records = $this->{records}->{$esn};
 
     # Issue requests for any records not already available or requested.
     for (my $i = 0; $i < $size; $i++) {
@@ -520,11 +507,11 @@ sub records {
 }
 
 
-=head2 errcode(), addinfo()
+=head2 errcode(), addinfo(), errmsg()
 
 	if (!defined $rs->record($n)) {
 		print "error number: ", $rs->errcode(), "\n";
-		print "additional info: ", $rs->errcode(), "\n";
+		print "additional info: ", $rs->addinfo(), "\n";
 	}
 
 When a result set's C<record()> method returns an undefined value,
@@ -532,6 +519,9 @@ indicating an error, it also sets into the result set the BIB-1 error
 code and additional information returned by the server.  They can be
 retrieved via the C<errcode()> and C<addinfo()> methods.
 
+As a convenience, C<$rs->errmsg()> is equivalent to
+C<Net::Z3950::errstr($rs->errcode())>.
+
 =cut
 
 sub errcode {
@@ -544,6 +534,11 @@ sub addinfo {
     return $this->{addinfo};
 }
 
+sub errmsg {
+    my $this = shift();
+    return Net::Z3950::errstr($this->errcode());
+}
+
 
 =head2 option()
 
